<?php

namespace Ebuy\Demo\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Ebuy\Demo\Http\Requests\DemoRequest;
use Ebuy\Demo\Interfaces\DemoRepositoryInterface;
use Ebuy\Demo\Models\Demo;

/**
 * Resource controller class for demo.
 */
class DemoResourceController extends BaseController
{

    /**
     * Initialize demo resource controller.
     *
     * @param type DemoRepositoryInterface $demo
     *
     * @return null
     */
    public function __construct(DemoRepositoryInterface $demo)
    {
        parent::__construct();
        $this->repository = $demo;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Ebuy\Demo\Repositories\Criteria\DemoResourceCriteria::class);
    }

    /**
     * Display a list of demo.
     *
     * @return Response
     */
    public function index(DemoRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Ebuy\Demo\Repositories\Presenter\DemoPresenter::class)
                ->$function();
        }

        $demos = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('demo::demo.names'))
            ->view('demo::demo.index', true)
            ->data(compact('demos', 'view'))
            ->output();
    }

    /**
     * Display demo.
     *
     * @param Request $request
     * @param Model   $demo
     *
     * @return Response
     */
    public function show(DemoRequest $request, Demo $demo)
    {

        if ($demo->exists) {
            $view = 'demo::demo.show';
        } else {
            $view = 'demo::demo.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('demo::demo.name'))
            ->data(compact('demo'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new demo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DemoRequest $request)
    {

        $demo = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('demo::demo.name')) 
            ->view('demo::demo.create', true) 
            ->data(compact('demo'))
            ->output();
    }

    /**
     * Create new demo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DemoRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $demo                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('demo::demo.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('demo/demo/' . $demo->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/demo/demo'))
                ->redirect();
        }

    }

    /**
     * Show demo for editing.
     *
     * @param Request $request
     * @param Model   $demo
     *
     * @return Response
     */
    public function edit(DemoRequest $request, Demo $demo)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('demo::demo.name'))
            ->view('demo::demo.edit', true)
            ->data(compact('demo'))
            ->output();
    }

    /**
     * Update the demo.
     *
     * @param Request $request
     * @param Model   $demo
     *
     * @return Response
     */
    public function update(DemoRequest $request, Demo $demo)
    {
        try {
            $attributes = $request->all();

            $demo->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('demo::demo.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('demo/demo/' . $demo->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('demo/demo/' . $demo->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the demo.
     *
     * @param Model   $demo
     *
     * @return Response
     */
    public function destroy(DemoRequest $request, Demo $demo)
    {
        try {

            $demo->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('demo::demo.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('demo/demo/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('demo/demo/' . $demo->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple demo.
     *
     * @param Model   $demo
     *
     * @return Response
     */
    public function delete(DemoRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('demo::demo.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('demo/demo'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/demo/demo'))
                ->redirect();
        }

    }

    /**
     * Restore deleted demos.
     *
     * @param Model   $demo
     *
     * @return Response
     */
    public function restore(DemoRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('demo::demo.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/demo/demo'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/demo/demo/'))
                ->redirect();
        }

    }

}
